package tfg;

import java.awt.Desktop;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.Arrays;
import java.util.Calendar;

import com.google.api.client.googleapis.auth.oauth2.GoogleAuthorizationCodeFlow;
import com.google.api.client.googleapis.auth.oauth2.GoogleClientSecrets;
import com.google.api.client.googleapis.auth.oauth2.GoogleCredential;
import com.google.api.client.googleapis.auth.oauth2.GoogleTokenResponse;
import com.google.api.client.http.HttpTransport;
import com.google.api.client.http.javanet.NetHttpTransport;
import com.google.api.client.json.JsonFactory;
import com.google.api.client.json.jackson.JacksonFactory;
import com.google.api.services.drive.DriveScopes;
import com.google.gdata.client.spreadsheet.CellQuery;
import com.google.gdata.client.spreadsheet.SpreadsheetService;
import com.google.gdata.data.spreadsheet.CellEntry;
import com.google.gdata.data.spreadsheet.CellFeed;
import com.google.gdata.data.spreadsheet.SpreadsheetEntry;
import com.google.gdata.data.spreadsheet.SpreadsheetFeed;
import com.google.gdata.data.spreadsheet.WorksheetEntry;
import com.google.gdata.util.ServiceException;


class GSpreadsheets {
	private static final String REDIRECT_URI = "urn:ietf:wg:oauth:2.0:oob";
	private static final String CLIENT_SECRET_PATH = "resources/client_secret.json";
	private static final String REFRESH_TOKEN_PATH = "resources/google.txt";

	private static java.util.List<SpreadsheetEntry> spreadsheets;
	private static SpreadsheetService service;
	private static int columna;
	private static GoogleClientSecrets clientSecrets;

	public static void openWebpage(URI uri) {
		Desktop desktop = Desktop.isDesktopSupported() ? Desktop.getDesktop() : null;
		if (desktop != null && desktop.isSupported(Desktop.Action.BROWSE)) {
			try {
				desktop.browse(uri);
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}

	public void inicializarGoogle() throws IOException, ServiceException {
//		// Get client ID and SECRET from https://cloud.google.com/console
//		// get credentials similar to Java DrEdit example
//		// https://developers.google.com/drive/examples/java
        HttpTransport httpTransport = new NetHttpTransport();
        JsonFactory jsonFactory = new JacksonFactory();

        clientSecrets = GoogleClientSecrets.load(jsonFactory,  new FileReader(CLIENT_SECRET_PATH));

        GoogleCredential credential = null;
        
        File f = new File(REFRESH_TOKEN_PATH);
        
        if(f.exists() && !f.isDirectory()) { // Si fichero con el refresh token ya existe se lee el token
        	BufferedReader br = new BufferedReader(new FileReader(REFRESH_TOKEN_PATH));
        	String tokenRefresh = "";
        	try {
                StringBuilder sb = new StringBuilder();
                String line = br.readLine();

                while (line != null) {
                    sb.append(line);
                    sb.append(System.lineSeparator());
                    line = br.readLine();
                }
                tokenRefresh = sb.toString();
            } finally {
                br.close();
            }
        	
        	credential = new GoogleCredential.Builder().setTransport(httpTransport)
        	        .setJsonFactory(jsonFactory)
        	        .setClientSecrets(clientSecrets)
        	        .build()
        	        .setRefreshToken(tokenRefresh);
        
        } else {  // Si no existe, se obtiene y se crea el fichero
    		GoogleAuthorizationCodeFlow flow = new GoogleAuthorizationCodeFlow.Builder(
    				httpTransport, jsonFactory, clientSecrets,
    				Arrays.asList(DriveScopes.DRIVE, 
    						"https://spreadsheets.google.com/feeds", 
    						"https://docs.google.com/feeds"))
    		.setAccessType("offline")
    		.setApprovalPrompt("auto").build();

    		String url = flow.newAuthorizationUrl().setRedirectUri(REDIRECT_URI).build();
    		System.out.println("Por favor, abra el siguiente enlace en su navegador "
    				+ "y luego introduzca aqui el código de autorización: ");

    		System.out.println("  " + url);
    	

    		URL enlace = new URL(url);
    		try {
    			openWebpage(enlace.toURI());
    		} catch (URISyntaxException e) {
    			e.printStackTrace();
    		}

    		BufferedReader br = new BufferedReader(new InputStreamReader(System.in));
    		String code = br.readLine();

    		GoogleTokenResponse response = flow.newTokenRequest(code).setRedirectUri(REDIRECT_URI).execute();
    		
    		PrintWriter writer = new PrintWriter(REFRESH_TOKEN_PATH, "UTF-8");
    	    writer.println(response.getAccessToken());
    	    writer.close();
    	    
    	    credential = new GoogleCredential.Builder().setTransport(httpTransport)
        	        .setJsonFactory(jsonFactory)
        	        .setClientSecrets(clientSecrets)
        	        .build()
        	        .setRefreshToken(response.getAccessToken());
        }
        

		// create the service and pass it the credentials you created earlier
		service = new SpreadsheetService("Marker Detector");
		service.setOAuth2Credentials(credential);

		// Define the URL to request.  This should never change.
		URL SPREADSHEET_FEED_URL = new URL(
				"https://spreadsheets.google.com/feeds/spreadsheets/private/full");

		// Make a request to the API and get all spreadsheets.
		SpreadsheetFeed feed = service.getFeed(SPREADSHEET_FEED_URL, SpreadsheetFeed.class);
		spreadsheets = feed.getEntries();

	}

	public void inicializarFecha() throws IOException, ServiceException {
		// Iterate through all of the spreadsheets returned
		for (SpreadsheetEntry spreadsheet : spreadsheets) {
			if (spreadsheet.getTitle().getPlainText().equalsIgnoreCase("listaAlumnos")) {
				// Print the title of this spreadsheet to the screen
				System.out.println(spreadsheet.getTitle().getPlainText());

				// Make a request to the API to fetch information about all
				// worksheets in the spreadsheet.
				java.util.List<WorksheetEntry> worksheets = spreadsheet.getWorksheets();

				for (WorksheetEntry worksheet : worksheets) {

					URL cellFeedUrl = worksheet.getCellFeedUrl();

					Calendar c = Calendar.getInstance();
					String dia = Integer.toString(c.get(Calendar.DATE)); 
					String mes = Integer.toString(c.get(Calendar.MONTH) + 1); 
					String annio = Integer.toString(c.get(Calendar.YEAR));
					String fecha;
					if (Integer.parseInt(mes) + 1 < 10) {
						fecha = dia + "/0" + mes + "/" + annio;
					} 
					else {
						fecha = dia + "/" + mes + "/" + annio;
					}

					CellQuery query = new CellQuery(cellFeedUrl);
					query.setMinimumRow(1);
					query.setMaximumRow(1);

					CellFeed feedQuery = service.query(query, CellFeed.class);

					int col = 1;
					String value = "";
					boolean fechaExiste = false;
					for (CellEntry entry : feedQuery.getEntries()) {
						value = entry.getCell().getInputValue();
					
						if (value.equalsIgnoreCase(fecha)) {
							columna = col;
							fechaExiste = true;
						}
						else {
							col++;
							columna = col;
						}
					}
					
					if (!fechaExiste) {
						CellEntry newEntry = new CellEntry(1, columna, fecha);
						service.insert(cellFeedUrl, newEntry); 
					} 
				}
			}
		}
	}

	public static void asisteAlumno(String niu) throws IOException, ServiceException {
		for (SpreadsheetEntry spreadsheet : spreadsheets) {
			if (spreadsheet.getTitle().getPlainText().equalsIgnoreCase("listaAlumnos")) {

				java.util.List<WorksheetEntry> worksheets = spreadsheet.getWorksheets();

				for (WorksheetEntry worksheet : worksheets) {

					URL cellFeedUrl = worksheet.getCellFeedUrl();

					CellQuery query = new CellQuery(cellFeedUrl);

					query.setMinimumCol(1);
					query.setMaximumCol(1);
					CellFeed feedQuery = service.query(query, CellFeed.class);

					int fila = 0;
					int cont = 1;
					String value = "";
					for (CellEntry entry : feedQuery.getEntries()) {

						value = "0" + entry.getCell().getInputValue();
						System.out.println(value);
						if (value.equalsIgnoreCase(niu)) {
							fila = cont;
						}
						cont++;  
					}

					CellEntry newEntry = new CellEntry(fila, columna, "Asiste");
					service.insert(cellFeedUrl, newEntry); 
				}
			}
		}
	}
}
